import json, socket
from base64 import b64encode
from .errors import ConnectionError, InvalidResponseError, api_errors

try:
    from urllib.request import urlopen, Request
    from urllib.error import HTTPError, URLError
    from urllib.parse import urlencode
except ImportError:
    from urllib2 import Request, urlopen, HTTPError, URLError
    from urllib import urlencode

class Pin(object):

    API_VERSION = '1'

    HOST = 'api.pin.net.au'
    TEST_HOST = 'test-api.pin.net.au'

    def __init__(self, secret, testing=False, host=None):
        self.auth = self.http_auth(secret, '')
        if host is not None:
            self.host = host
        elif testing:
            self.host = Pin.TEST_HOST
        else:
            self.host = Pin.HOST

    def http_auth(self, username, password):
        auth = '%s:%s' % (username, password)
        return 'Basic %s' % b64encode(auth.encode('ascii')).decode('ascii')

    def url_for(self, endpoint):
        return 'https://%s/%s%s' % (self.host, Pin.API_VERSION, endpoint)

    def request(self, method, endpoint, data=None):
        response = self.raw_request(method, endpoint, data)
        try:
            if 'error' in response:
                error_type, error = response['error'], response['error_description']
                if 'messages' in response:
                    messages = ( item['message'] for item in response['messages'] )
                    error += ' (%s)' % ', '.join(messages)
                raise api_errors[error_type](error)
            response = response['response']
        except (KeyError, ValueError, TypeError):
            raise InvalidResponseError
        return response

    def raw_request(self, method, endpoint, data=None):
        url = self.url_for(endpoint)
        if method == 'GET':
            if data is not None:
                url += '?' + urlencode(data)
            request = Request(url)
        else:
            request = Request(url, json.dumps(data or {}).encode('utf8'))
            request.add_header('Content-Type', 'application/json')
        request.add_header('Authorization', self.auth)
        request.get_method = lambda: method
        try:
            response = urlopen(request)
        except HTTPError as err:
            response = err
        except (URLError, socket.error) as e:
            raise ConnectionError(str(e))
        try:
            response = json.loads(response.read().decode('utf8'))
        except ValueError:
            raise InvalidResponseError
        return response

